#include <math.h>
#include "particle.h"

static float randFloat(struct drand48_data *rng) {
	double val;
	drand48_r(rng, &val);
	return (float)val;
}

static int randInt(struct drand48_data *rng) {
	long int val;
	mrand48_r(rng, &val);
	return (int)val;
}

void particle_data_alloc_buffers(ParticleData *pd, int num_particles)
{
	pd->x = (float *) malloc(sizeof(float) * num_particles);
	pd->y = (float *) malloc(sizeof(float) * num_particles);
	pd->dx = (float *) malloc(sizeof(float) * num_particles);
	pd->dy = (float *) malloc(sizeof(float) * num_particles);
	pd->mass = (float *) malloc(sizeof(float) * num_particles);
	pd->color = (int *) malloc(sizeof(int) * num_particles);
}

void particle_data_free_buffers(ParticleData *pd)
{
	free(pd->x);
	free(pd->y);
	free(pd->dx);
	free(pd->dy);
	free(pd->mass);
	free(pd->color);
}

void particle_init_rand(float *x, float *y, float *dx, float *dy, float *mass, int *color, int index, struct drand48_data *rng)
{
	float startWidth = WIDTH/4;
	float startHeight = HEIGHT/4;
	x[index] = (randFloat(rng) * startWidth) - (startWidth / 2);
	y[index] = (randFloat(rng) * startHeight) - (startHeight / 2);
	dx[index] = (randFloat(rng) * 50.0) - 25.0;
	dy[index] = (randFloat(rng) * 50.0) - 25.0;
	mass[index] = 128000.0;
	color[index] = randInt(rng) & 0xFFFFFF;
}

void particle_update_position(float *x, float *y, float *dx, float *dy, int index)
{
	// The .01 scaling factor here is to make the simulation "smoother",
	// by decreasing the distance the particles move per time step.
	x[index] += (dx[index] * .01);
	y[index] += (dy[index] * .01);
}
